;; sxwindow support functions and function keymap

;; GNU Emacs is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 1, or (at your option)
;; any later version.

;; GNU Emacs is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, 675 Mass Ave, Cambridge, MA 02139, USA.

;; 94.3.02 Created for Mule Ver.1.0 on sxwindow by T.Ishikura

;; TASK $B$N=*N;$r4F;k$9$k4X?t$r:n$i$J$1$l$P$J$i$J$$!#(B
;; $B$$$C$=$N$3$H!"(BSYSTEM1,2 EVENT $B$O(B lisp $B$N(B vector $B$+2?$+$G!"(B
;; $BBP1~$9$k(B lisp function $B$r8F$Y$k$h$&$K$7$h$&$+!#(B

(defun sx-execute-file (filename)
  "create a task on sxwindow. the task is determined from
file FILENAME using icon list."
  (interactive "ffilename (full path): ")
  (sx-execute-file-noconv
   (code-convert-string filename *internal* *sjis*)))

(defun sx-tsfock (filename &optional commandline)
  "execute a file FILENAME on sxwindow width COMMANDLINE and return 
taskID. Not use emacs environ, but use sxwindow environ."
  (sx-tsfock-noconv
   (code-convert-string filename *internal* *sjis*)
   (if commandline
       (code-convert-string commandline *internal* *sjis*))))

(defun sx-dired-execute ()
  (interactive)
  (if (eq window-system 'sx)
      (sx-execute-file (dired-get-filename))))

(defun get-window-local-pos-from-global-pos(win p)
  "Get position relative to WINDOW from global POSITION (x-char y-char)."
  (let ((windowedge (window-edges win)))
    (list
     (- (car p) (car windowedge))
     (- (car (cdr p)) (car (cdr windowedge))))))

(defun get-window-from-pos (p)
  "Get window in which position X (x-char y-char) is."
  (let ((windowlist (window-list))
	(x (car p))
	(y (car (cdr p)))
	(result))
    (while windowlist
      (let ((windowedge (window-edges (car windowlist))))
	(and
	 (<= (car windowedge) x)
	 (<= (car (cdr windowedge)) y)
	 (< x (car (cdr (cdr windowedge))))
	 (< y (car (cdr (cdr (cdr windowedge)))))
	 (setq result (car windowlist)))
	(setq windowlist (cdr windowlist))))
    result))

(defun sx-drag()
  (interactive)
  (let ((dragwin (get-window-from-pos sx-drag-position)))
    (cond (dragwin
	   (select-window dragwin)
	   (mapcar 'find-file sx-drag-file-list))
	  (t(error "Icon was dragged not on window contents.")))))

(defun sx-cursor-set-pos (p)
  "Select Emacs window POSITION (x-char y-char) is on,
and move point to POSITION."
  (let ((selectwindow (get-window-from-pos p))
	(localpos)
	(margin-column))
    (cond (selectwindow
	   (select-window selectwindow)
	   (setq localpos (get-window-local-pos-from-global-pos selectwindow p))
	   (let ((prompt-width (if (eq (selected-window) (minibuffer-window))
				   minibuffer-prompt-width 0))
		 target-column)
	     (move-to-window-line (car (cdr localpos)))
	     (if (eobp)
		 nil
	       (setq margin-column
		     (if (or truncate-lines (> (window-hscroll) 0))
			 (current-column)
		       (- (current-column)
			  (% (current-column) (1- (window-width))))))
	       (setq target-column (+ (car localpos) (1- (max 1 (window-hscroll)))
				      (if (= (point) 1)
					  (- prompt-width) 0)
				      margin-column))
	       (if (> (move-to-column target-column) target-column)
		   (forward-char -1)))))
	   (t (error "position is not on any window.")))))
  
;; $B<!$NFs$D$N4X?t$O$$$$$+$2$s$J$N$G!"$"$H$+$i$^$H$b$K$9$k!#(B

(defun sx-save-some-buffers (&optional arg exiting)
  "Save some modified file-visiting buffers.  Asks user about each one.
Optional argument (the prefix) non-nil means save all with no questions.
Optional second argument EXITING means ask about certain non-file buffers
 as well as about file buffers."
  (interactive "P")
  (let (considered (list (buffer-list)))
    (while list
      (let ((buffer (car list)))
	(and (buffer-modified-p buffer)
	     (save-excursion
	       (set-buffer buffer)
	       (and
		(or buffer-file-name
		    (and exiting buffer-offer-save (> (buffer-size) 0)))
		(setq considered t)
		(or arg
		    (sx-yes-or-no-p (if buffer-file-name
					(format "Save file %s? "
						buffer-file-name)
				      (format "Save buffer %s? " (buffer-name)))))
		(condition-case ()
		    (save-buffer)
		  (error nil))))))
      (setq list (cdr list)))
    (and save-abbrevs abbrevs-changed
	 (progn
	   (setq considered t)
	   (if (or arg
		   (sx-yes-or-no-p (format "Save abbrevs in %s? " abbrev-file-name)))
	       (write-abbrev-file nil))
	   ;; Don't keep bothering user if he says no.
	   (setq abbrevs-changed nil)))
    (if considered
	(message "")
      (message "(No files need saving)"))))

(defun sx-save-buffers-kill-emacs (&optional arg)
  "Offer to save each buffer, then kill this Emacs fork.
With prefix arg, silently save all file-visiting buffers, then kill."
  (interactive "P")
  (sx-save-some-buffers arg t)
  (and (or (not (memq t (mapcar (function
				  (lambda (buf) (and (buffer-file-name buf)
						     (buffer-modified-p buf))))
				(buffer-list))))
	   (sx-yes-or-no-p "Modified buffers exist; exit anyway? "))
       (or (not (fboundp 'process-list))
	   ;; process-list is not defined on VMS.
	   (let ((processes (process-list))
		 active)
	     (while processes
	       (and (memq (process-status (car processes)) '(run stop open))
		    (let ((val (process-kill-without-query (car processes))))
		      (process-kill-without-query (car processes) val)
		      val)
		    (setq active t))
	       (setq processes (cdr processes)))
	     (or (not active)
		 (sx-yes-or-no-p "Active processes exist; kill them and exit anyway? "))))
       (kill-emacs)))

(defun sx-mouse-click ()
  (interactive)
  (let ((mouse (sx-get-mouse-event)))
    (if (eq (car mouse) 'sx-mouse-click)
	(sx-cursor-set-pos (car (cdr mouse))))))

(defun sx-save-buffer-context ()
  (interactive)
  (setq save-context-filename "~/.emacs_SX")
  (save-context))

(defun sx-mouse-double-click ()
  (interactive)
  (if (eq (car (sx-get-mouse-event)) 'sx-double-click)
      (let ((point-save (point)))
	(forward-word 1)
	(if (< 1 (count-lines point-save (point)))
	    (goto-char point-save)
	  (push-mark nil t)
	  (sit-for 1)
	  (forward-word -1)))))

(defun sx-mouse-triple-click ()
  (interactive)
  (if (eq (car (sx-get-mouse-event)) 'sx-triple-click)
      (progn
	(forward-line 1)
	(push-mark nil t)
	(forward-char -1)
	(sit-for 1)
	(forward-line 0))))

(defun sx-mouse-drag ()
  (interactive)
  (let* ((start (car sx-mouse-drag-position))
	 (end (car (cdr sx-mouse-drag-position)))
	 (window (get-window-from-pos start)))
    (if (and window
	     (eq window (get-window-from-pos end)))
	(progn
	  (sx-cursor-set-pos end)
	  (push-mark nil t)
	  (sit-for 1)
	  (sx-cursor-set-pos start)))))

(defun sx-scroll-down ()
  (interactive)
  (let ((cwindow (selected-window)))
    (select-window sx-selected-scrollbar-window)
    (scroll-down 1)
    (select-window cwindow)))

(defun sx-scroll-up ()
  (interactive)
  (let ((cwindow (selected-window)))
    (select-window sx-selected-scrollbar-window)
    (scroll-up 1)
    (select-window cwindow)))

(defun sx-page-down ()
  (interactive)
  (let ((cwindow (selected-window)))
    (select-window sx-selected-scrollbar-window)
    (scroll-down nil)
    (select-window cwindow)))

(defun sx-page-up ()
  (interactive)
  (let ((cwindow (selected-window)))
    (select-window sx-selected-scrollbar-window)
    (scroll-up nil)
    (select-window cwindow)))

(defun sx-scroll-bar ()
  (interactive)
  (let ((cwindow (selected-window)))
    (select-window sx-selected-scrollbar-window)
    (goto-char sx-selected-scrollbar-value)
    (select-window cwindow)))

(defun sx-menu-copy ()
  (sx-put-to-scrap
   (code-convert-string 
    (buffer-substring (point) (mark))
    *internal* *sjis*dos)))

(defun sx-menu-cut ()
  (sx-menu-copy)
  (delete-region (point) (mark)))

(defun sx-menu-paste ()
  (insert-string (code-convert-string
		  (sx-get-from-scrap)
		  *sjis*dos *internal*)))

(defun sx-menu-select-all ()
  (goto-char (point-max))
  (sit-for 1)
  (push-mark nil t)
  (goto-char 1))

(defun sx-menu-split-window ()
  (let ((win (get-window-from-pos sx-right-down-position)))
    (if win
	(split-window win nil)
      (error "position is not on any window."))))

(defun sx-menu-delete-window ()
  (let ((win (get-window-from-pos sx-right-down-position)))
    (if win
	(delete-window win)
      (error "position is not on any window."))))

(defun sx-menu-enlarge-window ()
  (let ((win (get-window-from-pos sx-right-down-position))
	(cwin (selected-window)))
    (if win
	(progn
	  (select-window win)
	  (enlarge-window 1)
	  (select-window cwin))
      (error "position is not on any window."))))

(defun sx-menu-kill-buffer ()
  (let ((win (get-window-from-pos sx-right-down-position)))
    (if win
	(kill-buffer (window-buffer win))
      (error "position is not on any window."))))

(defun sx-mouse-right-down ()
  (interactive)
  (eval
   (sx-select-menu-noconv 
    '(("^XCut" . (sx-menu-cut))
      ("^CCopy" . (sx-menu-copy))
      ("^VPaste" . (sx-menu-paste))
      ("Split window" . (sx-menu-split-window))
      ("Delete window" . (sx-menu-delete-window))
      ("Enlarge window" . (sx-menu-enlarge-window))
      ("Kill buffer" . (sx-menu-kill-buffer))
      ("~" . nil)
      ("Select all" . (sx-menu-select-all))
      ("Undo" . (undo))))))

(defvar sx-sxbasic-message-hooks nil 
  "List of functions to be called after received sxbasic messages.")

(defun sx-send-sxbasic-message (s i)
  (sx-send-sxbasic-message-noconv
   (code-convert-string s *internal* *sjis*dos) i))

(defun sx-sxbasic-message-default ()
  (message "%s %s" (car (car sx-sxbasic-message-list))
	   (code-convert-string (car (cdr (car sx-sxbasic-message-list)))
				*sjis*dos *internal*))
  (setq sx-sxbasic-message-list
	(cdr sx-sxbasic-message-list)))

(defun sx-sxbasic-message ()
  (interactive)
  (if sx-sxbasic-message-hooks
      (run-hooks 'sx-sxbasic-message-hooks)
    (sx-sxbasic-message-default)))

;; Make X68-raw-map to handle X68000 terminal function keys
;; and sxwindow event.

(defvar X68-raw-map (make-sparse-keymap) "*Keymap for C-^ encoded keyboard")

(define-key X68-raw-map "A" 'beginning-of-buffer) ; F1
(define-key X68-raw-map "B" 'end-of-buffer)	  ; F2
(define-key X68-raw-map "C" 'replace-string)	  ; F3
(define-key X68-raw-map "D" 'isearch-forward)	  ; F4
(define-key X68-raw-map "E" "?????")		  ; F5
(define-key X68-raw-map "F" 'set-mark-command)    ; F6
(define-key X68-raw-map "G" 'kill-region)	  ; F7
(define-key X68-raw-map "H" 'copy-region-as-kill) ; F8
(define-key X68-raw-map "I" 'yank)		  ; F9
(define-key X68-raw-map "J" 'suspend-emacs)	  ; F10
(define-key X68-raw-map "a" 'shrink-window)	  ; F11
(define-key X68-raw-map "b" 'enlarge-window)	  ; F12
(define-key X68-raw-map "c" "?????")		  ; F13
(define-key X68-raw-map "d" 'isearch-backward)	  ; F14
(define-key X68-raw-map "e" "?????")		  ; F15
(define-key X68-raw-map "f" "?????")		  ; F16
(define-key X68-raw-map "g" "?????")		  ; F17
(define-key X68-raw-map "h" "?????")		  ; F18
(define-key X68-raw-map "i" "?????")		  ; F19
(define-key X68-raw-map "j" "?????")		  ; F20
(define-key X68-raw-map "K" 'scroll-up)	          ; ROLLUP
(define-key X68-raw-map "L" 'scroll-down)         ; ROLLDOWN
(define-key X68-raw-map "M" "?????")		  ; INS
(define-key X68-raw-map "N" 'delete-char)	  ; DEL
(define-key X68-raw-map "O" 'previous-line)	  ; UP
(define-key X68-raw-map "P" 'backward-char)	  ; LEFT
(define-key X68-raw-map "Q" 'forward-char)	  ; RIGHT
(define-key X68-raw-map "R" 'next-line)	  	  ; DOWN
(define-key X68-raw-map "T" 'help-for-help)	  ; HELP
(define-key X68-raw-map "U" 'other-window)	  ; HOME
(define-key X68-raw-map "V" 'undo)		  ; UNDO

;; sxwindow event

(define-key X68-raw-map "0" 'sx-save-buffers-kill-emacs); ENDTSK, CLOSEALL
(define-key X68-raw-map "1" 'sx-save-buffer-context)   	; SAVE
(define-key X68-raw-map "2" 'sx-drag)			; DRAG
(define-key X68-raw-map "3" 'sx-save-buffers-kill-emacs); CLOSE BUTTOM
;;(define-key X68-raw-map "4" 'sx-mouse-left-down)	; MOUSE LEFT DOWN
(define-key X68-raw-map "5" 'sx-mouse-right-down)	; MOUSE RIGHT DOWM
(define-key X68-raw-map "6" 'sx-mouse-click)		; MOUSE LEFT CLICK
(define-key X68-raw-map "7" 'sx-mouse-double-click)	; MOUSE LEFT BOUBLE CLICK
(define-key X68-raw-map "8" 'sx-mouse-triple-click)	; MOUSE LEFT TRIPLE CLICK
(define-key X68-raw-map "9" 'sx-mouse-drag)		; MOUSE DRAG

(define-key X68-raw-map "!" 'sx-save-buffers-kill-emacs); Opt1 + Q
(define-key X68-raw-map "#" 'sx-menu-paste)		; Opt1 + V
(define-key X68-raw-map "$" 'sx-menu-copy)		; Opt1 + C
(define-key X68-raw-map "%" 'sx-menu-cut)		; Opt1 + X
(define-key X68-raw-map "&" 'save-buffer)		; Opt1 + S

(define-key X68-raw-map "z" 'sx-scroll-down)		; SCROLLBAR SCROLL DOWN
(define-key X68-raw-map "y" 'sx-scroll-up)	       	; SCROLLBAR SCROLL UP
(define-key X68-raw-map "x" 'sx-page-down)		; SCROLLBAR PAGE DOWN
(define-key X68-raw-map "w" 'sx-page-up)		; SCROLLBAR PAGE UP
(define-key X68-raw-map "v" 'sx-scroll-bar)		; SCROLLBAR DRAG
(define-key X68-raw-map "u" 'sx-sxbasic-message)	; GET SXBASIC MESSAGE

;; $B$3$l0J9_$O%a%K%e!<4X78$NDj5A!#$3$l$O$"$H$G40A4$KJQ99$5$l$^$9!#(B

;;(define-key X68-raw-map "k" 'sx-menu-cut)
;;(define-key X68-raw-map "l" 'sx-menu-copy)
;;(define-key X68-raw-map "m" 'sx-menu-paste)
;;(define-key X68-raw-map "n" 'sx-menu-select-all)
;;(define-key X68-raw-map "o" 'sx-menu-undo)

(define-key global-map "\C-^" X68-raw-map)		; Install X68-raw-map

